#include <iostream>
#include <sctl.hpp>

int main(int argc, char** argv) {
  sctl::Comm::MPI_Init(&argc, &argv);
  sctl::Profile::Enable(true);
  constexpr int DIM = 3;
  using Real = double;

  if (1) {
    sctl::Comm comm = sctl::Comm::World();
    srand48(comm.Rank());

    sctl::Vector<Real> coord, coord_unif;
    { // Set coord, coord_unif
      long N_total = 2e5;
      int np = comm.Size();
      int myrank = comm.Rank();
      long start= myrank    * N_total / np;
      long end  =(myrank+1) * N_total / np;
      long N_local = end - start;
      coord.ReInit(N_local * DIM);
      for(long i=0;i<N_local;i++){
        Real phi = 2 * M_PI * drand48();
        Real theta = M_PI * drand48();
        coord[i*DIM+0] = 0.5 + 0.1125 * sin(theta) * cos(phi);
        coord[i*DIM+1] = 0.5 + 0.1125 * sin(theta) * sin(phi);
        coord[i*DIM+2] = 0.5 + 0.45 * cos(theta);
      }

      coord_unif.ReInit(1000);
      for (auto& x : coord_unif) x = drand48();
    }

    sctl::Profile::Tic("Refine",&comm);
    sctl::PtTree<Real,DIM> t(comm);
    t.UpdateRefinement(coord_unif, 1000, true, false);
    sctl::Profile::Toc();

    sctl::Profile::Tic("AddPts",&comm);
    t.AddParticles("src_pts", coord);
    sctl::Profile::Toc();

    sctl::Profile::Tic("UpdateRefine",&comm);
    t.UpdateRefinement(coord, 100, true, false);
    sctl::Profile::Toc();

    sctl::Profile::Tic("GetPts",&comm);
    { // Verify GetParticleData
      sctl::Vector<Real> data;
      t.GetParticleData(data, "src_pts");
      SCTL_ASSERT(data.Dim() == coord.Dim());
      for (long i = 0; i < coord.Dim(); i++) SCTL_ASSERT(data[i] == coord[i]);
    }
    sctl::Profile::Toc();

    t.WriteTreeVTK("tree");
    t.WriteParticleVTK("pts", "src_pts");

    t.DeleteParticleData("src_pts");

    sctl::Profile::print(&comm);
  }

  sctl::Comm::MPI_Finalize();
  return 0;
}
